import javax.media.opengl.GL;

import util.StringTools;

/**
 * Materials provided by sample program at:<br>
 * http://ezekiel.vancouver.wsu.edu/~cs442/archive/lectures/shading/tea.c
 */
public enum Material {
  BLACK_PLASTIC (
      new float[] {0.0f, 0.0f, 0.0f, 1.0f}, 
      new float[] {0.01f, 0.01f, 0.01f, 1.0f}, 
      new float[] {0.5f, 0.5f, 0.5f, 1.0f}, 
      32.0f),
  BRASS (
      new float[] {0.329412f, 0.223529f, 0.027451f, 1.0f},
      new float[] {0.780392f, 0.568627f, 0.113725f, 1.0f},
      new float[] {0.992157f, 0.941176f, 0.807843f, 1.0f},
      27.8974f),
  BRONZE (
      new float[] {0.2125f, 0.1275f, 0.054f, 1.0f},
      new float[] {0.714f, 0.4284f, 0.18144f, 1.0f},
      new float[] {0.393548f, 0.271906f, 0.166721f, 1.0f},
      25.6f),
  CHROME (
      new float[] {0.25f, 0.25f, 0.25f, 1.0f},
      new float[] {0.4f, 0.4f, 0.4f, 1.0f},
      new float[] {0.774957f, 0.774957f, 0.774957f, 1.0f},
      76.8f),
  COPPER (
      new float[] {0.19125f, 0.0735f, 0.0225f, 1.0f},
      new float[] {0.7038f, 0.27048f, 0.0828f, 1.0f},
      new float[] {0.256777f, 0.137622f, 0.086014f, 1.0f},
      12.8f),
  GOLD (
      new float[] {0.24725f, 0.1995f, 0.0745f, 1.0f},
      new float[] {0.75164f, 0.60648f, 0.22648f, 1.0f},
      new float[] {0.628281f, 0.555802f, 0.366025f, 1.0f},
      51.2f),
  PEWTER (
      new float[] {0.10588f, 0.058824f, 0.113725f, 1.0f},
      new float[] {0.427451f, 0.470588f, 0.541176f, 1.0f},
      new float[] {0.3333f, 0.3333f, 0.521569f, 1.0f},
      9.84615f),
  SILVER (
      new float[] {0.19225f, 0.19225f, 0.19225f, 1.0f},
      new float[] {0.50754f, 0.50754f, 0.50754f, 1.0f},
      new float[] {0.508273f, 0.508273f, 0.508273f, 1.0f},
      51.2f),
  POLISHED_SILVER (
      new float[] {0.23125f, 0.23125f, 0.23125f, 1.0f},
      new float[] {0.2775f, 0.2775f, 0.2775f, 1.0f},
      new float[] {0.773911f, 0.773911f, 0.773911f, 1.0f},
      89.6f),
  BLUE_PLASTIC (
      new float[] {0.1f, 0.1f, 1f, 1.0f},
      new float[] {0.1f, 0.1f, 1f, 1.0f},
      new float[] {0.5f, 0.5f, 1f, 1.0f},
      51.2f);
  
  float[] ambient, diffuse, specular;
  float exponent;
  
  /**
   * Returns the enum representation of a string. This is case sensitive.
   * @param str toString representation of this enum
   * @return enum associated with a string
   * @throws IllegalArgumentException if argument is not represented by this enum.
   */
  public static Material fromString(String str) {
    for (Material type : Material.values()) {
      if (str.equals(type.toString())) return type;
    }
    throw new IllegalArgumentException();
  }
  
  Material(float[] ambient, float[] diffuse, float[] specular, float exponent) {
    this.ambient = ambient;
    this.diffuse = diffuse;
    this.specular = specular;
    this.exponent = exponent;
  }
  
  public void applyTexture(GL gl) {
    gl.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, this.ambient, 0);
    gl.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, this.diffuse, 0);
    gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, this.specular, 0);
    gl.glMaterialf(GL.GL_FRONT, GL.GL_SHININESS, this.exponent);
  }
  
  @Override
  public String toString() {
    return StringTools.getReadableConstant(this.name());
  }
}
